/*-
 * Copyright (C) 2007 Erik Larsson
 * 
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package org.catacombae.findduplicates;

import org.catacombae.findduplicates.gui.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ProgressDialog extends JDialog implements ProgressListener {
    private static final String gatheringFilesString = "Gathering all files from directories...";
    private static final String sizeMatchString = "Matching file sizes...";
    private static final String hashString = "Calculating hash values where needed...";
    private ProgressPanel pPanel;
    private String currentMessage = null;
    private ProgressListener.CancelHandler cancelHandler = null;

    public ProgressDialog(Frame owner) {
	super(owner, "Progress...", true);
	pPanel = new ProgressPanel();
	
	// Initialize some aspects of pPanel
	pPanel.progressBar.setMinimum(0);
	pPanel.progressBar.setMaximum(Integer.MAX_VALUE);
	pPanel.progressBar.setStringPainted(true);
	pPanel.cancelButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if(cancelHandler != null)
			cancelHandler.actionCancel();
		}
	    });

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent we) {
		    if(cancelHandler != null)
			cancelHandler.actionCancel();
		}
	    });
	
	add(pPanel);
	pack();
	setLocationRelativeTo(null);
	setResizable(false);
	reset();
    }
    
    public void reset() {
	pPanel.progressBar.setValue(0);
	currentMessage = gatheringFilesString;
	pPanel.statusLabel.setText(gatheringFilesString);
    }

    /** <pre> progress is supposed to be in the range 0.0 <= progress <= 1.0 </pre> */
    public void reportGatherFilesProgress(double progress) {
	final int min = 0;
	final int max = Integer.MAX_VALUE/10;
	final int range = max-min;
	pPanel.progressBar.setValue(min+(int)(range*progress));
	if(currentMessage != gatheringFilesString) {
	    currentMessage = gatheringFilesString;
	    pPanel.statusLabel.setText(currentMessage);
	}
    }
    // This is currently not used as a progress monitor, and its range has been taken over by reportGatherFilesProgress
    public void reportSizeMatchProgress(double progress) {
// 	final int min = Integer.MAX_VALUE/20;
// 	final int max = Integer.MAX_VALUE/10;
// 	final int range = max-min;
// 	pPanel.progressBar.setValue(min+(int)(range*progress));
	if(currentMessage != sizeMatchString) {
	    currentMessage = sizeMatchString;
	    pPanel.statusLabel.setText(currentMessage);
	}
    }
    /** <pre> progress is supposed to be in the range 0.0 <= progress <= 1.0 </pre> */
    public void reportHashProgress(double progress) {
	final int min = Integer.MAX_VALUE/10+1;
	final int max = Integer.MAX_VALUE;
	final int range = max-min;
	pPanel.progressBar.setValue(min+(int)(range*progress));
	if(currentMessage != hashString) {
	    currentMessage = hashString;
	    pPanel.statusLabel.setText(currentMessage);
	}
    }

    public void setCancelHandler(ProgressListener.CancelHandler handler) {
	cancelHandler = handler;
    }

    public void reportNonCriticalError(String message) {
	JOptionPane.showMessageDialog(this, message, "Non critical error", JOptionPane.ERROR_MESSAGE);
    }
}
